﻿#ifndef engine_device_manager_h
#define engine_device_manager_h

#include "engine_utils.h"
#include "engine_types.h"

namespace ding {
namespace rtc {

/** \defgroup CPP_DingRtcEngineVideoDevice 视频设备相关接口
 *  DingRtc 视频设备相关接口
 */

/** \defgroup CPP_DingRtcEngineAudioDevice 音频设备相关接口
 *  DingRtc 音频设备相关接口
 */

//! \addtogroup Basic_Datatypes  数据类型参考
//! @{

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 设备类型。
 */
typedef enum {
  /** 未知类型。 */
  RtcEngineTypeUnknown = -1,
  /** 音频采集设备。 */
  RtcEngineTypeAudioRecording = 0,
  /** 音频播放设备。 */
  RtcEngineTypeAudioPlayout = 1,
  /** 视频采集设备。 */
  RtcEngineTypeVideoCapture = 2,
} RtcEngineDeviceType;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 设备状态。
 */
typedef enum {
  /** 设备已添加。 */
  RtcEngineDeviceStateAdd = 0,
  /** 设备已移除。 */
  RtcEngineDeviceStateRemove = 1,
} RtcEngineDeviceState;

/**
 * @ingroup CPP_DingRtcDef
 * @since 3.0
 * @brief 设备信息。
 */
typedef struct {
  /** 设备名称。 */
  String deviceName;
  /** 设备ID。 */
  String deviceID;
} RtcEngineDeviceInfo;

#if defined(ALI_OS_ANDROID) || defined(ALI_OS_IOS)
typedef enum {
  /** 默认音频路由 */
  kRtcEngineAudioRouteDefault = 0,
  /** 耳机 */
  kRtcEngineAudioRouteHeadset = 1,
  /** 听筒 */
  kRtcEngineAudioRouteEarpiece = 2,
  /** 不带麦的耳机 */
  kRtcEngineAudioRouteHeadsetNoMic = 3,
  /** 手机扬声器 */
  kRtcEngineAudioRouteSpeakerphone = 4,
  /** 外接扬声器 */
  kRtcEngineAudioRouteLoudSpeaker = 5,
  /** 蓝牙耳机 */
  kRtcEngineAudioRouteBluetooth = 6
} RtcEngineAudioRouteType;
#endif

/**
 * @brief 设备列表。
 */
class DING_RTC_API RtcEngineDeviceInfoList {
public:
  virtual unsigned int GetCount() = 0;
  virtual RtcEngineDeviceInfo GetDeviceInfo(unsigned int index) = 0;
  virtual void Release() = 0;

public:
  virtual ~RtcEngineDeviceInfoList() {}
};

typedef void (*CameraListCallBack)(RtcEngineDeviceInfoList* camera_list);

/**
 * @brief 音频设备相关功能。
*/
class DING_RTC_API RtcEngineAudioDeviceManager {
public:
    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 获取系统中的录音设备列表。
     * @return 系统中的录音设备列表。
     */
    virtual RtcEngineDeviceInfoList* GetRecordingDeviceList() = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 获取使用的录音设备id。
     * @return 当前音频采集设备id。
     */
    virtual String GetCurrentRecordingDeviceId() = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 选择录音设备id。
     * @param deviceId 音频采集设备id。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetCurrentRecordingDeviceId(const char* deviceId) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 获取系统中的扬声器列表。
     * @return 音频播放设备列表。
     */
    virtual RtcEngineDeviceInfoList* GetPlayoutDeviceList() = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 获取当前使用的扬声器id。
     * @return 当前音频播放设备id。
     */
    virtual String GetCurrentPlayoutDeviceId() = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 选择扬声器id。
     * @param deviceId 音频播放设备id。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetCurrentPlayoutDeviceId(const char* deviceId) = 0;

    /**
     * @brief 设置音频采集设备音量，音量范围[0, 100]。
     * @param volume 音量值。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetRecordingDeviceVolume(int volume) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 获取音频采集设备音量。
     * @return
     * - >=0：音量；
     * - <0：失败。
     */
    virtual int GetRecordingDeviceVolume() = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 设置音频播放设备音量，音量范围[0, 100]。
     * @param volume 音量值。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetPlayoutDeviceVolume(int volume) = 0;

    /**
     * @brief 获取音频播放设备音量。
     * @return 设备音量。
     */
    virtual int GetPlayoutDeviceVolume() = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 静音音频采集设备。
     * @param mute 
     * - 是否静音。
     * - true：设备设置为静音。
     * - false：设备设置为不静音。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetRecordingDeviceMute(bool mute) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 使用静音帧替换采集到的数据。
     * @param mute 
     * - 是否用静音帧代替采集到的音频数据。
     * - true：使用静音帧。
     * - false：不使用静音帧。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetRecordingDataMute(bool mute) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 获取音频采集设备静音状态。
     * @return
     * - true：已静音状态；
     * - false：非静音状态。
     */
    virtual bool GetRecordingDeviceMute() = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 静音音频播放设备。
     * @param mute 
     * - 是否静音。
     * - true：设备设置为静音。
     * - false：设备设置为不静音。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetPlayoutDeviceMute(bool mute) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 获取音频播放设备静音状态。
     * @return
     * - true：已静音状态；
     * - false：非静音状态。
     */
    virtual bool GetPlayoutDeviceMute() = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 开启音频采集设备测试。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note 开始测试后音量信息通过 {@link RtcEngineEventListener::OnRecordingDeviceAudioLevel} 回调返回。
     */
    virtual int StartRecordingDeviceTest() = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 停止音频采集设备测试。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int StopRecordingDeviceTest() = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 开启音频播放设备测试。
     * @param waveFilePath wave文件路径。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note 开始测试后音量信息通过 {@link RtcEngineEventListener::OnPlayoutDeviceAudioLevel} 回调返回。
     */
    virtual int StartPlayoutDeviceTest(const char* waveFilePath, int channels, int samplerate) = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 停止测试音频播放设备。
     * @return
     * - 0：成功；
     * - 非0：失败。
    */
    virtual int StopPlayoutDeviceTest() = 0;

    /**
     * @ingroup CPP_DingRtcEngineAudioDevice
     * @since 3.0
     * @brief 开启/关闭音频设备音量观测。
     * @param deviceType 音频设备类型。
     * @param enable 
     *          - true表示开启音量观测。
     *          - false表示关闭音量观测。
     * @return
     * - 0：成功；
     * - 非0：失败。
     * @note 开启音量观测，会分别通过{@link RtcEngineEventListener::OnRecordingDeviceAudioLevel}和
     *        {@link RtcEngineEventListener::OnPlayoutDeviceAudioLevel}回调采集和播放设备音量值。
     */
    virtual int EnableAudioLevelObserver(ding::rtc::RtcEngineDeviceType deviceType, bool enable) = 0;

#if defined(ALI_OS_ANDROID) || defined(ALI_OS_IOS)
    /**
     * @brief 设置默认音频输出是否从扬声器出声，默认从扬声器出声。
     * @param defaultToSpeakerphone true:扬声器模式(默认扬声器)；false:听筒模式。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetDefaultAudioRouteToSpeakerphone(bool defaultToSpeakerphone) = 0;

    /**
     * @brief 设置音频输出为听筒还是扬声器（默认扬声器）。
     * @param enable true:扬声器模式；false:听筒模式。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int EnableSpeakerphone(bool enable) = 0;

    /**
     * @brief 获取是否已开启扬声器。
     * @return
     * - true：已开启扬声器；
     * - false：未开启扬声器。
     */
    virtual bool IsSpeakerphoneEnabled() = 0;
#endif

protected:
    virtual ~RtcEngineAudioDeviceManager() {}
};

/**
 * @brief 视频设备相关功能。
 */
class DING_RTC_API RtcEngineVideoDeviceManager
{
public:
    /**
     * @ingroup CPP_DingRtcEngineVideoDevice
     * @since 3.0
     * @brief 获取摄像头列表。
     * @return {@link RtcEngineDeviceInfoList} 设备列表。
     */
    virtual RtcEngineDeviceInfoList* GetCameraList() = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideoDevice
     * @since 3.0
     * @brief 异步获取摄像头列表。
     * @return 0：调用成功；其他：调用失败。
     */
    virtual int GetCameraList(CameraListCallBack callback) = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideoDevice
     * @since 3.0
     * @brief 获取当前使用的摄像头名称。
     * @return 摄像头名称。
     */
    virtual String GetCurrentCameraName() = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideoDevice
     * @since 3.0
     * @brief 获取当前使用的摄像头ID。
     * @return 摄像头ID。
     */
    virtual String GetCurrentCameraID() = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideoDevice
     * @since 3.0
     * @brief 选择摄像头名称。
     * @param cameraName 摄像头名称。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetCurrentCameraName(const char* cameraName) = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideoDevice
     * @since 3.0
     * @brief 选择摄像头ID。
     * @param cameraID 摄像头ID。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetCurrentCameraID(const char* cameraID) = 0;

    /**
     * @ingroup CPP_DingRtcEngineVideoDevice
     * @since 3.0
     * @brief 摄像头是否开启。
     * @return
     * - true：摄像头已经开启
     * - false：摄像头没有开启
     */
    virtual bool IsCameraOn() = 0;

#if defined(ALI_OS_IOS) || defined(ALI_OS_ANDROID)
    /**
     * @brief 选择摄像头方向。
     * @param direction 摄像头方向。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetCurrentCameraDirection(RtcEngineCameraDirection direction) = 0;
    
    /**
     * @brief 切换前后摄。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SwitchCamera() = 0;

    /**
     * @brief 设置摄像头zoom。
     * @param zoom 缩放级别，范围从 1 到 camera支持的最大 zoom值。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetCameraZoom(float zoom) = 0;

    /**
     * @brief 设置摄像头闪光灯开关。
     * @param flash 是否打开闪光灯。
     * @return
     * - 0：设置成功；
     * - 非0：设置失败。
     */
    virtual int SetCameraFlash(bool enabled) = 0;

    /**
     * @brief 是否支持摄像头手动聚焦。
     * @return
     * - true：支持；
     * - false：不支持。
     */
    virtual bool IsCameraFocusPointSupported() = 0;

    /**
     * @brief 设置摄像头手动聚焦点。
     * @param x x轴坐标值。
     * @param y y轴坐标值。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetCameraFocusPoint(float x, float y) = 0;

    /**
     * @brief 是否支持摄像头手动聚焦。
     * @return
     * - true：支持；
     * - false：不支持。
     */
    virtual bool IsCameraExposurePointSupported() = 0;

    /**
     * @brief 设置摄像头曝光点。
     * @param x x轴坐标值。
     * @param y y轴坐标值。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual int SetCameraExposurePoint(float x, float y) = 0;

    /**
     * @brief 设置采集方向。
     * @param orientation 采集的方向，0竖屏，90左横屏，180上下颠倒，270右横屏，其他：由SDK自己决定。
     * @return
     * - 0：成功；
     * - 非0：失败。
     */
    virtual void SetDeviceOrientationMode(int orientation) = 0;
#endif

protected:
    virtual ~RtcEngineVideoDeviceManager() {}
};

//! @}

} // namespace rtc
} // namespace ding


#endif /* engine_device_manager_h */
